#include <iostream>
#include <iterator>
#include <list>
#include <vector>

template <class Iterator>                    // erste überladene Funktion
auto holeMittleres(Iterator anfang, int n, 
                  std::input_iterator_tag)
{
  for (int i = 0; i < n / 2; ++i) {          // bis zur Mitte laufen
    ++anfang;
  }
  return *anfang;
}
// Der bidirektionale Iterator lässt keine wahlfreien Zugriffe und damit
// keine Iteratorarithmetik zu. Zur Bewegung sind nur die Operatoren
// ++  und -- erlaubt. Ein Random-Access-Iterator erlaubt
// Arithmetik, sodass die Implementation für diesen Fall etwas einfacher ist:
template <class Iterator>                    // zweite überladene Funktion
auto holeMittleres(Iterator anfang, int n, 
                  std::random_access_iterator_tag)
{
  return *(anfang + n / 2);                  // Arithmetik
}

// Aufrufende Funktion.
// Diese Funktion ruft nun die korrespondierende überladene Variante auf,
// wobei die Auswahl zur Compilierzeit durch den Parameter kategorie geschieht.
template <class Iterator> 
auto mittleresElement(Iterator anfang, int n)
{
  typename std::iterator_traits<Iterator>::iterator_category kategorie;
  return holeMittleres(anfang, n, kategorie);
}

int main()                                   // Hauptprogramm
{
  std::list<int> liste{0, 1, 2, 3, 4, 5, 6, 7, 8, 9};          // list
  std::vector<int> vec{0, 1, 2, 3, 4, 5, 6, 7, 8, 9};          // vector
  // Aufruf der ersten Implementierung für die Liste
  std::cout << mittleresElement(liste.begin(), liste.size()) << '\n';
  // Aufruf der zweiten Implementierung für den Vektor
  std::cout << mittleresElement(vec.begin(), vec.size()) << '\n';
}

