#include <cassert>
#include <fstream>
#include <iostream>
#include <source_location>
#include <string>
#include <system_error>
#include <vector>
using namespace std;

template <typename T> 
using Matrix = std::vector<std::vector<T>>;  // zur Abkürzung

template <typename T>
void writeVectorMatrix(const std::string& filename, const Matrix<T>& matrix)
{
  std::ofstream dest(filename, std::ios::binary);
  if (!dest) {
    const auto& wo {source_location::current()};
    throw  std::ios::failure(std::string(wo.file_name()) + ", Zeile " 
                             + std::to_string(wo.line()));
  }
  for (const auto& zeile : matrix) {         // zeilenweise schreiben
    dest.write(reinterpret_cast<const char*>(zeile.data()),
                                             zeile.size() * sizeof(T));
  }
}

template <typename T>
void readVectorMatrix(const std::string& filename, Matrix<T>& matrix)
{
  std::ifstream source(filename, std::ios::binary);
  if (!source) {
    const auto& wo {source_location::current()};
    throw  std::ios::failure(std::string(wo.file_name()) + ", Zeile " 
                             + std::to_string(wo.line()));
  }
  for (auto& zeile : matrix) {               // zeilenweise lesen
    source.read(reinterpret_cast<char*>(zeile.data()), 
                                        zeile.size() * sizeof(T));
  }
}

int main()
{
  try {
    std::string dateiname("binaerdatenMatrix.bin");
    Matrix<double> mat1 = {{
        {{1.1, 2.2, 3.3}}, 
        {{4.4, 5.5, 6.6}}
      }};
    writeVectorMatrix(dateiname, mat1);          // mat1 schreiben

    // neue Matrix mat2 in der richtigen Größe anlegen:
    Matrix<double> mat2(mat1.size(),                           // Zeilenzahl
                        std::vector<double>(mat1[0].size()));  // damit vorbesetzen
    readVectorMatrix(dateiname, mat2);                         // mat2 lesen
    assert(mat1 == mat2);                                      // auf Gleichheit prüfen
  }
  catch(const std::ios::failure& exc) {
    cerr << exc.what() << '\n';
  }
}

